from autoarray.masked.masked_structures import MaskedArray as array, MaskedGrid as grid
from autoastro.galaxy.masked.galaxy_data import MaskedGalaxyData as masked_galaxy_data
from autolens.masked.masked_dataset import (
    MaskedImaging as imaging,
    MaskedInterferometer as interferometer,
)
